# coding: utf-8

"""
FastAPI

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

from truefoundry.pydantic_v1 import BaseModel, StrictInt, StrictStr, conlist


class SearchRunsRequestDto(BaseModel):
    """
    SearchRunsRequestDto
    """

    experiment_ids: Optional[conlist(StrictStr)] = None
    filter: Optional[StrictStr] = None
    run_view_type: Optional[StrictStr] = "ACTIVE_ONLY"
    max_results: Optional[StrictInt] = 1000
    order_by: Optional[conlist(StrictStr)] = None
    page_token: Optional[StrictStr] = None
    offset: Optional[StrictInt] = None
    __properties = [
        "experiment_ids",
        "filter",
        "run_view_type",
        "max_results",
        "order_by",
        "page_token",
        "offset",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SearchRunsRequestDto:
        """Create an instance of SearchRunsRequestDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SearchRunsRequestDto:
        """Create an instance of SearchRunsRequestDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SearchRunsRequestDto.parse_obj(obj)

        _obj = SearchRunsRequestDto.parse_obj(
            {
                "experiment_ids": obj.get("experiment_ids"),
                "filter": obj.get("filter"),
                "run_view_type": obj.get("run_view_type")
                if obj.get("run_view_type") is not None
                else "ACTIVE_ONLY",
                "max_results": obj.get("max_results")
                if obj.get("max_results") is not None
                else 1000,
                "order_by": obj.get("order_by"),
                "page_token": obj.get("page_token"),
                "offset": obj.get("offset"),
            }
        )
        return _obj
