# coding: utf-8

"""
FastAPI

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class MultiPartUploadStorageProvider(str, Enum):
    """
    An enumeration.
    """

    """
    allowed enum values
    """
    S3_COMPATIBLE = "S3_COMPATIBLE"
    AZURE_BLOB = "AZURE_BLOB"

    @classmethod
    def from_json(cls, json_str: str) -> MultiPartUploadStorageProvider:
        """Create an instance of MultiPartUploadStorageProvider from a JSON string"""
        return MultiPartUploadStorageProvider(json.loads(json_str))
