# coding: utf-8

"""
FastAPI

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class SklearnSerializationFormat(str, Enum):
    """
    +label=Serialization format +usage=Serialization format used for sklearn models
    """

    """
    allowed enum values
    """
    CLOUDPICKLE = "cloudpickle"
    JOBLIB = "joblib"
    PICKLE = "pickle"

    @classmethod
    def from_json(cls, json_str: str) -> SklearnSerializationFormat:
        """Create an instance of SklearnSerializationFormat from a JSON string"""
        return SklearnSerializationFormat(json.loads(json_str))
