# coding: utf-8

"""
FastAPI

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional, Union

from truefoundry.pydantic_v1 import BaseModel, Field, StrictFloat, StrictInt, StrictStr


class LogMetricRequestDto(BaseModel):
    """
    LogMetricRequestDto
    """

    run_id: Optional[StrictStr] = None
    run_uuid: Optional[StrictStr] = None
    key: StrictStr = Field(...)
    value: Union[StrictFloat, StrictInt] = Field(...)
    timestamp: StrictInt = Field(...)
    step: Optional[StrictInt] = 0
    __properties = ["run_id", "run_uuid", "key", "value", "timestamp", "step"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LogMetricRequestDto:
        """Create an instance of LogMetricRequestDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LogMetricRequestDto:
        """Create an instance of LogMetricRequestDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LogMetricRequestDto.parse_obj(obj)

        _obj = LogMetricRequestDto.parse_obj(
            {
                "run_id": obj.get("run_id"),
                "run_uuid": obj.get("run_uuid"),
                "key": obj.get("key"),
                "value": obj.get("value"),
                "timestamp": obj.get("timestamp"),
                "step": obj.get("step") if obj.get("step") is not None else 0,
            }
        )
        return _obj
